@@ -91,7 +91,7 @@ main(int argc, const char *argv[])
 
     /* Find HTML nodes by CSS Selectors. */
 
-    body = lxb_dom_interface_node(lxb_html_document_body_element(document));
+    body = lxb_dom_interface_node(document);
 
     printf("Found:\n");
 
