@@ -133,6 +133,13 @@ static const lxb_css_syntax_cb_declarations_t lxb_css_stylesheet_declarations =
 };
 
 
+lxb_css_stylesheet_t *
+lxb_css_stylesheet_create(lxb_css_memory_t *memory)
+{
+    return (lxb_css_stylesheet_t *) lexbor_mraw_calloc(memory->mraw,
+                                                       sizeof(lxb_css_stylesheet_t));
+}
+
 lxb_css_stylesheet_t *
 lxb_css_stylesheet_destroy(lxb_css_stylesheet_t *sst, bool destroy_memory)
 {
@@ -174,12 +181,6 @@ lxb_css_stylesheet_prepare(lxb_css_parser_t *parser, lxb_css_memory_t *memory,
     parser->tkz->with_comment = false;
     parser->stage = LXB_CSS_PARSER_RUN;
 
-    parser->chunk_cb = parser->tkz->chunk_cb;
-    parser->chunk_ctx = parser->tkz->chunk_ctx;
-
-    parser->tkz->chunk_cb = lxb_css_syntax_parser_tkz_cb;
-    parser->tkz->chunk_ctx = parser;
-
     return LXB_STATUS_OK;
 }
 
@@ -222,9 +223,6 @@ lxb_css_stylesheet_process(lxb_css_parser_t *parser,
 void
 lxb_css_stylesheet_finish(lxb_css_parser_t *parser)
 {
-    parser->tkz->chunk_cb = parser->chunk_cb;
-    parser->tkz->chunk_ctx = parser->chunk_ctx;
-
     parser->memory = parser->old_memory;
     parser->selectors = parser->old_selectors;
     parser->stage = LXB_CSS_PARSER_END;
@@ -321,7 +319,9 @@ lxb_css_stylesheet_list_rules_end(lxb_css_parser_t *parser,
                                   const lxb_css_syntax_token_t *token,
                                   void *ctx, bool failed)
 {
-    lxb_css_rule_list_append(ctx, parser->context);
+    if (parser->context != NULL) {
+        lxb_css_rule_list_append(ctx, parser->context);
+    }
 
     parser->context = ctx;
 
@@ -389,6 +389,8 @@ lxb_css_stylesheet_at_rule_end(lxb_css_parser_t *parser,
         return LXB_STATUS_OK;
     }
 
+    parser->status = LXB_STATUS_OK;
+
     /* Custom rule can't be here. */
 
     /* Always true. */
@@ -443,7 +445,7 @@ lxb_css_stylesheet_qualified_rule_block(lxb_css_parser_t *parser,
 
     lxb_css_parser_current_rule(parser)->context = list;
 
-    if (token->type == LXB_CSS_SYNTAX_TOKEN__TERMINATED) {
+    if (token->type == LXB_CSS_SYNTAX_TOKEN__END) {
         return lxb_css_parser_success(parser);
     }
 
@@ -505,6 +507,7 @@ lxb_css_stylesheet_qualified_rule_end(lxb_css_parser_t *parser,
     }
 
     parser->context = bad;
+    parser->status = LXB_STATUS_OK;
 
     return LXB_STATUS_OK;
 }
@@ -656,7 +659,7 @@ lxb_css_stylesheet_declarations_bad(lxb_css_parser_t *parser,
     lxb_css_rule_declaration_t *declar = ctx;
     const lxb_css_syntax_declarations_offset_t *offset;
 
-    while (token != NULL && token->type != LXB_CSS_SYNTAX_TOKEN__TERMINATED) {
+    while (token != NULL && token->type != LXB_CSS_SYNTAX_TOKEN__END) {
         lxb_css_syntax_parser_consume(parser);
         token = lxb_css_syntax_parser_token(parser);
     }
