@@ -11,8 +11,9 @@
 
 
 typedef struct {
-    lxb_unicode_idna_cb_f cb;
-    void                  *context;
+    lxb_unicode_idna_cb_f   cb;
+    void                    *context;
+    lxb_unicode_idna_flag_t flags;
 }
 lxb_unicode_idna_ctx_t;
 
@@ -65,6 +66,10 @@ lxb_unicode_idna_to_unicode_body(lxb_unicode_idna_t *idna, const void *data,
                                  void *ctx, lxb_unicode_idna_flag_t flags,
                                  bool is_cp);
 
+static bool
+lxb_unicode_idna_validity_criteria_h(const void *data, size_t length,
+                                     lxb_unicode_idna_flag_t flags, bool is_cp);
+
 lxb_unicode_idna_t *
 lxb_unicode_idna_create(void)
 {
@@ -90,6 +95,10 @@ lxb_unicode_idna_clean(lxb_unicode_idna_t *idna)
 lxb_unicode_idna_t *
 lxb_unicode_idna_destroy(lxb_unicode_idna_t *idna, bool self_destroy)
 {
+    if (idna == NULL) {
+        return NULL;
+    }
+
     (void) lxb_unicode_normalizer_destroy(&idna->normalizer, false);
 
     if (self_destroy) {
@@ -111,13 +120,15 @@ lxb_unicode_idna_realloc(lxb_codepoint_t *buf, const lxb_codepoint_t *buffer,
  
     if (buf == buffer) {
         tmp = lexbor_malloc(nlen * sizeof(lxb_codepoint_t));
+        if (tmp == NULL) {
+            return NULL;
+        }
     }
     else {
         tmp = lexbor_realloc(buf, nlen * sizeof(lxb_codepoint_t));
-    }
-
-    if (tmp == NULL) {
-        return NULL;
+        if (tmp == NULL) {
+            return lexbor_free(buf);
+        }
     }
 
     *buf_p = tmp + (*buf_p - buf);
@@ -156,7 +167,7 @@ lxb_unicode_idna_processing_body(lxb_unicode_idna_t *idna, const void *data,
     lxb_codepoint_t cp, *buf, *buf_p, *buf_end;
     const lxb_char_t *end, *p;
     lxb_unicode_idna_type_t type;
-    const lxb_unicode_data_t *udata;
+    const lxb_unicode_idna_entry_t *udata;
     const lxb_codepoint_t *maps;
     lxb_unicode_idna_ctx_t context;
     lxb_codepoint_t buffer[4096];
@@ -191,15 +202,14 @@ lxb_unicode_idna_processing_body(lxb_unicode_idna_t *idna, const void *data,
                 break;
 
             case LXB_UNICODE_IDNA_MAPPED:
-                udata = lxb_unicode_data(cp);
-                maps = udata->idna->cps;
-                length = udata->idna->length;
+                udata = lxb_unicode_idna_entry_by_cp(cp);
+                maps = lxb_unicode_idna_map(udata, &length);
 
                 if (buf_p + length > buf_end) {
                     buf = lxb_unicode_idna_realloc(buf, buffer, &buf_p,
                                                    &buf_end, length);
                     if (buf == NULL) {
-                        goto failed;
+                        return LXB_STATUS_ERROR_MEMORY_ALLOCATION;
                     }
                 }
 
@@ -209,27 +219,6 @@ lxb_unicode_idna_processing_body(lxb_unicode_idna_t *idna, const void *data,
 
                 break;
 
-            case LXB_UNICODE_IDNA_DISALLOWED_STD3_MAPPED:
-                if (!(flags & LXB_UNICODE_IDNA_FLAG_USE_STD3ASCII_RULES)) {
-                    type = LXB_UNICODE_IDNA_MAPPED;
-                }
-                else {
-                    type = LXB_UNICODE_IDNA_DISALLOWED;
-                }
-
-                goto again;
-
-            case LXB_UNICODE_IDNA_DISALLOWED_STD3_VALID:
-                if (!(flags & LXB_UNICODE_IDNA_FLAG_USE_STD3ASCII_RULES)) {
-                    type = LXB_UNICODE_IDNA_VALID;
-                    goto again;
-                }
-                else {
-                    type = LXB_UNICODE_IDNA_DISALLOWED;
-                }
-
-                goto again;
-
             case LXB_UNICODE_IDNA_DEVIATION:
                 if ((flags & LXB_UNICODE_IDNA_FLAG_TRANSITIONAL_PROCESSING)) {
                     type = LXB_UNICODE_IDNA_MAPPED;
@@ -247,7 +236,7 @@ lxb_unicode_idna_processing_body(lxb_unicode_idna_t *idna, const void *data,
                     buf = lxb_unicode_idna_realloc(buf, buffer, &buf_p,
                                                    &buf_end, 1);
                     if (buf == NULL) {
-                        goto failed;
+                        return LXB_STATUS_ERROR_MEMORY_ALLOCATION;
                     }
                 }
 
@@ -258,6 +247,8 @@ lxb_unicode_idna_processing_body(lxb_unicode_idna_t *idna, const void *data,
 
     context.cb = cb;
     context.context = ctx;
+    context.flags = flags;
+
 
     need = lxb_unicode_quick_check_cp(&idna->normalizer, buf, buf_p - buf,
                                       true);
@@ -279,14 +270,6 @@ done:
     }
 
     return status;
-
-failed:
-
-    if (buf != buffer) {
-        (void) lexbor_free(buf);
-    }
-
-    return LXB_STATUS_ERROR_MEMORY_ALLOCATION;
 }
 
 static lxb_status_t
@@ -335,11 +318,12 @@ lxb_unicode_idna_norm_c_send(const lxb_codepoint_t *cps,
                              const lxb_codepoint_t *p,
                              lxb_unicode_idna_ctx_t *context)
 {
+    bool cr;
     lxb_status_t status;
 
     /* xn-- or Xn-- or xN-- or XN-- */
 
-    if (p - cps > 4
+    if (p - cps >= 4
         && (cps[0] == 0x0078 || cps[0] == 0x0058)
         && (cps[1] == 0x006E || cps[1] == 0x004E)
         && cps[2] == 0x002D && cps[3] == 0x002D)
@@ -358,13 +342,25 @@ lxb_unicode_idna_norm_c_send(const lxb_codepoint_t *cps,
         status = LXB_STATUS_OK;
     }
 
+    cr = lxb_unicode_idna_validity_criteria_cp(cps, p - cps, context->flags);
+    if (!cr) {
+        return LXB_STATUS_ERROR_UNEXPECTED_RESULT;
+    }
+
     return context->cb(cps, p - cps, context->context, status);
 }
 
 static lxb_status_t
 lxb_unicode_idna_punycode_cb(const lxb_codepoint_t *cps, size_t len, void *ctx)
 {
+    bool cr;
     lxb_unicode_idna_ctx_t *context = ctx;
+    lxb_unicode_idna_ascii_ctx_t *asc = context->context;
+
+    cr = lxb_unicode_idna_validity_criteria_cp(cps, len, asc->flags);
+    if (!cr) {
+        return LXB_STATUS_ERROR_UNEXPECTED_RESULT;
+    }
 
     return context->cb(cps, len, context->context, LXB_STATUS_OK);
 }
@@ -451,18 +447,12 @@ static lxb_status_t
 lxb_unicode_idna_ascii_puny_cb(const lxb_char_t *data, size_t length, void *ctx,
                                bool unchanged)
 {
-    bool cr;
     size_t nlen;
     lxb_char_t *tmp;
     lxb_unicode_idna_ascii_ctx_t *asc = ctx;
 
     static const lexbor_str_t prefix = lexbor_str("xn--");
 
-    cr = lxb_unicode_idna_validity_criteria(data, length, asc->flags);
-    if (!cr) {
-        return LXB_STATUS_ERROR_UNEXPECTED_RESULT;
-    }
-
     if (asc->p + length + 6 > asc->end) {
         nlen = ((asc->end - asc->buf) * 4) + length + 6;
 
@@ -500,33 +490,93 @@ bool
 lxb_unicode_idna_validity_criteria(const lxb_char_t *data, size_t length,
                                    lxb_unicode_idna_flag_t flags)
 {
+    return lxb_unicode_idna_validity_criteria_h(data, length, flags, false);
+}
+
+bool
+lxb_unicode_idna_validity_criteria_cp(const lxb_codepoint_t *data, size_t length,
+                                      lxb_unicode_idna_flag_t flags)
+{
+    return lxb_unicode_idna_validity_criteria_h(data, length, flags, true);
+}
+
+static bool
+lxb_unicode_idna_validity_criteria_h(const void *data, size_t length,
+                                     lxb_unicode_idna_flag_t flags, bool is_cp)
+{
+    size_t len;
     lxb_codepoint_t cp;
+    const lxb_codepoint_t *cps;
     const lxb_char_t *p, *end;
     lxb_unicode_idna_type_t type;
 
     p = data;
-    end = data + length;
+    len = length * ((is_cp) ? sizeof(lxb_codepoint_t) : 1);
+    end = (const lxb_char_t *) data + len;
 
     if (flags & LXB_UNICODE_IDNA_FLAG_CHECK_HYPHENS) {
         /* U+002D HYPHEN-MINUS */
 
-        if (length >= 4) {
-            if (p[3] == 0x002D || p[4] == 0x002D) {
+        if (is_cp) {
+            cps = data;
+
+            if (length > 4) {
+                if (cps[3] == 0x002D || cps[4] == 0x002D) {
+                    return false;
+                }
+            }
+
+            if (length >= 1) {
+                if (cps[0] == 0x002D || cps[length - 1] == 0x002D) {
+                    return false;
+                }
+            }
+        }
+        else {
+            if (length > 4) {
+                if (p[3] == 0x002D || p[4] == 0x002D) {
+                    return false;
+                }
+            }
+
+            if (length >= 1) {
+                if (p[0] == 0x002D || p[-1] == 0x002D) {
+                    return false;
+                }
+            }
+        }
+    }
+    else if (length >= 4) {
+        if (is_cp) {
+            cps = data;
+
+            if (   (cps[0] == 0x0078 || cps[0] == 0x0058)
+                && (cps[1] == 0x006E || cps[1] == 0x004E)
+                &&  cps[2] == 0x002D && cps[3] == 0x002D)
+            {
                 return false;
             }
         }
-
-        if (length >= 1) {
-            if (p[0] == 0x002D || end[-1] == 0x002D) {
+        else {
+            if (   (p[0] == 0x0078 || p[0] == 0x0058)
+                && (p[1] == 0x006E || p[1] == 0x004E)
+                &&  p[2] == 0x002D && p[3] == 0x002D)
+            {
                 return false;
             }
         }
     }
 
     while (p < end) {
-        cp = lxb_encoding_decode_valid_utf_8_single(&p, end);
-        if (cp == LXB_ENCODING_DECODE_ERROR) {
-            return false;
+        if (!is_cp) {
+            cp = lxb_encoding_decode_valid_utf_8_single(&p, end);
+            if (cp == LXB_ENCODING_DECODE_ERROR) {
+                return false;
+            }
+        }
+        else {
+            cp = *((const lxb_codepoint_t *) p);
+            p = (const lxb_char_t *) ((const lxb_codepoint_t *) p + 1);
         }
 
         /* U+002E ( . ) FULL STOP */
@@ -541,13 +591,6 @@ lxb_unicode_idna_validity_criteria(const lxb_char_t *data, size_t length,
             case LXB_UNICODE_IDNA_VALID:
                 break;
 
-            case LXB_UNICODE_IDNA_DISALLOWED_STD3_VALID:
-                if (!(flags & LXB_UNICODE_IDNA_FLAG_USE_STD3ASCII_RULES)) {
-                    break;
-                }
-
-                return false;
-
             case LXB_UNICODE_IDNA_DEVIATION:
                 if (!(flags & LXB_UNICODE_IDNA_FLAG_TRANSITIONAL_PROCESSING)) {
                     break;
@@ -558,7 +601,6 @@ lxb_unicode_idna_validity_criteria(const lxb_char_t *data, size_t length,
             case LXB_UNICODE_IDNA_DISALLOWED:
             case LXB_UNICODE_IDNA_IGNORED:
             case LXB_UNICODE_IDNA_MAPPED:
-            case LXB_UNICODE_IDNA_DISALLOWED_STD3_MAPPED:
             default:
                 return false;
         }
