@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2021 Alexander Borisov
+ * Copyright (C) 2019-2025 Alexander Borisov
  *
  * Author: Alexander Borisov <borisov@lexbor.com>
  */
@@ -32,7 +32,7 @@ typedef struct {
 chunk_ctx_t;
 
 typedef lxb_status_t
-(*parse_data_cb_f)(helper_t *helper, lexbor_str_t *str);
+(*parse_data_cb_f)(helper_t *helper, lexbor_str_t *str, bool with_range);
 
 
 static lxb_status_t
@@ -49,14 +49,7 @@ static lxb_status_t
 check_entry(helper_t *helper, unit_kv_value_t *entry, parse_data_cb_f cb);
 
 static lxb_status_t
-parse_data_new_tkz_cb(helper_t *helper, lexbor_str_t *str);
-
-static lxb_status_t
-parse_data_new_tkz_chunk_cb(helper_t *helper, lexbor_str_t *str);
-
-static lxb_status_t
-chunk_cb(lxb_css_syntax_tokenizer_t *tkz, const lxb_char_t **data,
-         const lxb_char_t **end, void *ctx);
+parse_data_new_tkz_cb(helper_t *helper, lexbor_str_t *str, bool with_range);
 
 static lxb_status_t
 check_token(helper_t *helper, unit_kv_value_t *entry,
@@ -218,17 +211,6 @@ check(helper_t *helper, unit_kv_value_t *value)
 
         lexbor_array_clean(&helper->tokens);
         lxb_css_syntax_tokenizer_clean(helper->tkz);
-
-        TEST_PRINTLN("Test #"LEXBOR_FORMAT_Z" chunks", (i + 1));
-
-        status = check_entry(helper, entries->list[i],
-                             parse_data_new_tkz_chunk_cb);
-        if (status != LXB_STATUS_OK) {
-            return status;
-        }
-
-        lexbor_array_clean(&helper->tokens);
-        lxb_css_syntax_tokenizer_clean(helper->tkz);
     }
 
     return LXB_STATUS_OK;
@@ -237,10 +219,11 @@ check(helper_t *helper, unit_kv_value_t *value)
 static lxb_status_t
 check_entry(helper_t *helper, unit_kv_value_t *entry, parse_data_cb_f cb)
 {
+    bool with_range;
     lxb_status_t status;
     lexbor_str_t *str;
     unit_kv_array_t *token_entries;
-    unit_kv_value_t *data, *tokens;
+    unit_kv_value_t *data, *tokens, *range;
 
     /* Validate */
     data = unit_kv_hash_value_nolen_c(entry, "data");
@@ -269,10 +252,24 @@ check_entry(helper_t *helper, unit_kv_value_t *entry, parse_data_cb_f cb)
         return print_error(helper, tokens);
     }
 
+    with_range = false;
+
+    range = unit_kv_hash_value_nolen_c(entry, "range");
+    if (range != NULL) {
+        if (unit_kv_is_bool(range)) {
+            with_range = unit_kv_bool(range);
+        }
+        else {
+            TEST_PRINTLN("Parameter 'range' must be an BOOL");
+
+            return print_error(helper, range);
+        }
+    }
+
     /* Parse */
     str = unit_kv_string(data);
 
-    status = cb(helper, str);
+    status = cb(helper, str, with_range);
 
     if (status != LXB_STATUS_OK) {
         TEST_PRINTLN("Failed to CSS");
@@ -333,7 +330,7 @@ check_raw(lxb_css_syntax_token_t *token)
 }
 
 static lxb_status_t
-parse_data_new_tkz_cb(helper_t *helper, lexbor_str_t *str)
+parse_data_new_tkz_cb(helper_t *helper, lexbor_str_t *str, bool with_range)
 {
     lxb_status_t status;
     lxb_css_syntax_token_t *token;
@@ -350,6 +347,7 @@ parse_data_new_tkz_cb(helper_t *helper, lexbor_str_t *str)
         return status;
     }
 
+    lxb_css_syntax_tokenizer_with_unicode_range(helper->tkz, with_range);
     helper->tkz->with_comment = true;
 
     lxb_css_syntax_tokenizer_buffer_set(helper->tkz, str->data, str->length);
@@ -374,70 +372,6 @@ parse_data_new_tkz_cb(helper_t *helper, lexbor_str_t *str)
     return LXB_STATUS_OK;
 }
 
-static lxb_status_t
-parse_data_new_tkz_chunk_cb(helper_t *helper, lexbor_str_t *str)
-{
-    chunk_ctx_t ctx;
-    lxb_status_t status;
-    lxb_css_syntax_token_t *token;
-
-    if (helper->tkz != NULL) {
-        helper->tkz = lxb_css_syntax_tokenizer_destroy(helper->tkz);
-    }
-
-    helper->tkz = lxb_css_syntax_tokenizer_create();
-    status = lxb_css_syntax_tokenizer_init(helper->tkz);
-    if (status != LXB_STATUS_OK) {
-        helper->tkz = lxb_css_syntax_tokenizer_destroy(helper->tkz);
-
-        return status;
-    }
-
-    helper->tkz->with_comment = true;
-
-    ctx.begin = str->data;
-    ctx.end = str->data + str->length;
-    ctx.ch = *str->data;
-
-    lxb_css_syntax_tokenizer_buffer_set(helper->tkz, &ctx.ch, 1);
-
-    lxb_css_syntax_tokenizer_chunk_cb_set(helper->tkz, chunk_cb, &ctx);
-
-    do {
-        token = lxb_css_syntax_token_next(helper->tkz);
-        if (token == NULL) {
-            return helper->tkz->status;
-        }
-
-        if (lxb_css_syntax_token_type(token) != LXB_CSS_SYNTAX_TOKEN__EOF) {
-            status = lexbor_array_push(&helper->tokens, token);
-            if (status != LXB_STATUS_OK) {
-                return status;
-            }
-        }
-    }
-    while (lxb_css_syntax_token_type(token) != LXB_CSS_SYNTAX_TOKEN__EOF);
-
-    return LXB_STATUS_OK;
-}
-
-static lxb_status_t
-chunk_cb(lxb_css_syntax_tokenizer_t *tkz, const lxb_char_t **data,
-         const lxb_char_t **end, void *ctx)
-{
-    chunk_ctx_t *chunk = ctx;
-
-    chunk->begin++;
-    chunk->ch = *chunk->begin;
-
-    if (chunk->begin < chunk->end) {
-        *data = &chunk->ch;
-        *end = *data + 1;
-    }
-
-    return LXB_STATUS_OK;
-}
-
 static lxb_status_t
 check_token(helper_t *helper, unit_kv_value_t *entry,
             lxb_css_syntax_token_t *token)
